Normandy.Agents.ToolCallResponse (normandy v0.2.0)

Response schema for agent outputs that may include tool calls.

This schema allows the LLM to either provide a final text response or request tool executions.

Summary

Types

t()

@type t() :: %Normandy.Agents.ToolCallResponse{
  __meta__: term(),
  content: String.t() | nil,
  tool_calls: [Normandy.Components.ToolCall.t()]
}