Normandy.Components.ToolCall (normandy v0.2.0)
View SourceRepresents a tool call request from the LLM.
Tool calls contain the name of the tool to execute and the input parameters.
Represents a tool call request from the LLM.
Tool calls contain the name of the tool to execute and the input parameters.