Normandy.Components.DateTimeProvider (normandy v0.2.0)
View SourceContext provider that supplies current date and time information to agents.
This provider can be registered with an agent to automatically include current date/time information in the system prompt.
Example
# Create a provider instance
provider = %Normandy.Components.DateTimeProvider{}
# Register with an agent
agent = BaseAgent.register_context_provider(agent, "datetime", provider)The provider will add a section to the system prompt like:
## Current Date and Time
2025-10-27 15:30:45.123456Z