Normandy.Components.SystemPromptGenerator (normandy v0.2.0)
View SourceGenerates structured system prompts for agents from prompt specifications.
Summary
Functions
Uses an instance Normandy.Components.SystemPromptSpecification to generate a system prompt for the agent.
Functions
@spec generate_prompt(Normandy.Components.PromptSpecification.t(), any()) :: String.t()
Uses an instance Normandy.Components.SystemPromptSpecification to generate a system prompt for the agent.
Example
spec = %{
background: ["you are a helpful assistant", "you assist me in assisting"],
steps: ["step 1", "step 2", "step 3"],
output_instructions: ["print the result", "as a result"]
}
Normandy.Components.generate_prompt(spec)Parameters
- prompt_specification: The prompt specification
- tool_registry: Optional tool registry for including tool information