Normandy.Metadata (normandy v0.2.0)
View SourceMetadata attached to schema structs.
Tracks the state, source, and context information for schema instances. Used internally by the Schema system to manage struct lifecycle.
Metadata attached to schema structs.
Tracks the state, source, and context information for schema instances. Used internally by the Schema system to manage struct lifecycle.